@@ -108,7 +108,7 @@ typedef struct rd_kafka_Node_s {
         int id;        /*< Node id */
         char *host;    /*< Node host */
         uint16_t port; /*< Node port */
-        char *rack_id; /*< (optional) Node rack id */
+        char *rack;    /*< (optional) Node rack id */
 } rd_kafka_Node_t;
 
 rd_kafka_Node_t *rd_kafka_Node_new(int32_t id,
@@ -116,8 +116,16 @@ rd_kafka_Node_t *rd_kafka_Node_new(int32_t id,
                                    uint16_t port,
                                    const char *rack_id);
 
+rd_kafka_Node_t *rd_kafka_Node_new_from_brokers(
+    int32_t id,
+    const struct rd_kafka_metadata_broker *brokers_sorted,
+    const rd_kafka_metadata_broker_internal_t *brokers_internal,
+    int broker_cnt);
+
 rd_kafka_Node_t *rd_kafka_Node_copy(const rd_kafka_Node_t *src);
 
 void rd_kafka_Node_destroy(rd_kafka_Node_t *node);
 
+void rd_kafka_Node_free(void *node);
+
 #endif /* _RDKAFKA_AUX_H_ */
