@@ -1,7 +1,8 @@
 /*
  * librdkafka - Apache Kafka C library
  *
- * Copyright (c) 2012,2013 Magnus Edenhill
+ * Copyright (c) 2012-2022, Magnus Edenhill
+ *               2023, Confluent Inc.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -52,6 +53,7 @@
 #include "rdkafka_partition.h"
 #include "rdkafka_offset.h"
 #include "rdkafka_broker.h"
+#include "rdkafka_request.h"
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -261,7 +263,7 @@ rd_kafka_offset_file_commit(rd_kafka_toppar_t *rktp) {
         rd_kafka_topic_t *rkt = rktp->rktp_rkt;
         int attempt;
         rd_kafka_resp_err_t err = RD_KAFKA_RESP_ERR_NO_ERROR;
-        int64_t offset          = rktp->rktp_stored_offset;
+        int64_t offset          = rktp->rktp_stored_pos.offset;
 
         for (attempt = 0; attempt < 2; attempt++) {
                 char buf[22];
@@ -322,7 +324,7 @@ rd_kafka_offset_file_commit(rd_kafka_toppar_t *rktp) {
                              rktp->rktp_partition, offset,
                              rktp->rktp_offset_path);
 
-                rktp->rktp_committed_offset = offset;
+                rktp->rktp_committed_pos.offset = offset;
 
                 /* If sync interval is set to immediate we sync right away. */
                 if (rkt->rkt_conf.offset_store_sync_interval_ms == 0)
@@ -528,7 +530,7 @@ rd_kafka_offset_broker_commit_cb(rd_kafka_t *rk,
                      rktp->rktp_rkt->rkt_topic->str, rktp->rktp_partition,
                      rktpar->offset, err ? "not " : "", rd_kafka_err2str(err));
 
-        rktp->rktp_committing_offset = 0;
+        rktp->rktp_committing_pos.offset = 0;
 
         rd_kafka_toppar_lock(rktp);
         if (rktp->rktp_flags & RD_KAFKA_TOPPAR_F_OFFSET_STORE_STOPPING)
@@ -539,6 +541,9 @@ rd_kafka_offset_broker_commit_cb(rd_kafka_t *rk,
 }
 
 
+/**
+ * @locks_required rd_kafka_toppar_lock(rktp) MUST be held.
+ */
 static rd_kafka_resp_err_t
 rd_kafka_offset_broker_commit(rd_kafka_toppar_t *rktp, const char *reason) {
         rd_kafka_topic_partition_list_t *offsets;
@@ -548,18 +553,21 @@ rd_kafka_offset_broker_commit(rd_kafka_toppar_t *rktp, const char *reason) {
         rd_kafka_assert(rktp->rktp_rkt->rkt_rk,
                         rktp->rktp_flags & RD_KAFKA_TOPPAR_F_OFFSET_STORE);
 
-        rktp->rktp_committing_offset = rktp->rktp_stored_offset;
+        rktp->rktp_committing_pos = rktp->rktp_stored_pos;
 
         offsets = rd_kafka_topic_partition_list_new(1);
         rktpar  = rd_kafka_topic_partition_list_add(
             offsets, rktp->rktp_rkt->rkt_topic->str, rktp->rktp_partition);
-        rktpar->offset = rktp->rktp_committing_offset;
+
+        rd_kafka_topic_partition_set_from_fetch_pos(rktpar,
+                                                    rktp->rktp_committing_pos);
+        rd_kafka_topic_partition_set_metadata_from_rktp_stored(rktpar, rktp);
 
         rd_kafka_dbg(rktp->rktp_rkt->rkt_rk, TOPIC, "OFFSETCMT",
-                     "%.*s [%" PRId32 "]: committing offset %" PRId64 ": %s",
+                     "%.*s [%" PRId32 "]: committing %s: %s",
                      RD_KAFKAP_STR_PR(rktp->rktp_rkt->rkt_topic),
-                     rktp->rktp_partition, rktp->rktp_committing_offset,
-                     reason);
+                     rktp->rktp_partition,
+                     rd_kafka_fetch_pos2str(rktp->rktp_committing_pos), reason);
 
         rd_kafka_commit0(rktp->rktp_rkt->rkt_rk, offsets, rktp,
                          RD_KAFKA_REPLYQ(rktp->rktp_ops, 0),
@@ -580,21 +588,20 @@ rd_kafka_offset_broker_commit(rd_kafka_toppar_t *rktp, const char *reason) {
  */
 static rd_kafka_resp_err_t rd_kafka_offset_commit(rd_kafka_toppar_t *rktp,
                                                   const char *reason) {
-        if (1)  // FIXME
-                rd_kafka_dbg(
-                    rktp->rktp_rkt->rkt_rk, TOPIC, "OFFSET",
-                    "%s [%" PRId32
-                    "]: commit: "
-                    "stored offset %" PRId64 " > committed offset %" PRId64 "?",
-                    rktp->rktp_rkt->rkt_topic->str, rktp->rktp_partition,
-                    rktp->rktp_stored_offset, rktp->rktp_committed_offset);
+        rd_kafka_dbg(rktp->rktp_rkt->rkt_rk, TOPIC, "OFFSET",
+                     "%s [%" PRId32 "]: commit: stored %s > committed %s?",
+                     rktp->rktp_rkt->rkt_topic->str, rktp->rktp_partition,
+                     rd_kafka_fetch_pos2str(rktp->rktp_stored_pos),
+                     rd_kafka_fetch_pos2str(rktp->rktp_committed_pos));
 
         /* Already committed */
-        if (rktp->rktp_stored_offset <= rktp->rktp_committed_offset)
+        if (rd_kafka_fetch_pos_cmp(&rktp->rktp_stored_pos,
+                                   &rktp->rktp_committed_pos) <= 0)
                 return RD_KAFKA_RESP_ERR_NO_ERROR;
 
         /* Already committing (for async ops) */
-        if (rktp->rktp_stored_offset <= rktp->rktp_committing_offset)
+        if (rd_kafka_fetch_pos_cmp(&rktp->rktp_stored_pos,
+                                   &rktp->rktp_committing_pos) <= 0)
                 return RD_KAFKA_RESP_ERR__PREV_IN_PROGRESS;
 
         switch (rktp->rktp_rkt->rkt_conf.offset_store_method) {
@@ -630,6 +637,8 @@ rd_kafka_resp_err_t rd_kafka_offset_sync(rd_kafka_toppar_t *rktp) {
  * Typically called from application code.
  *
  * NOTE: No locks must be held.
+ *
+ * @deprecated Use rd_kafka_offsets_store().
  */
 rd_kafka_resp_err_t rd_kafka_offset_store(rd_kafka_topic_t *app_rkt,
                                           int32_t partition,
@@ -637,6 +646,8 @@ rd_kafka_resp_err_t rd_kafka_offset_store(rd_kafka_topic_t *app_rkt,
         rd_kafka_topic_t *rkt = rd_kafka_topic_proper(app_rkt);
         rd_kafka_toppar_t *rktp;
         rd_kafka_resp_err_t err;
+        rd_kafka_fetch_pos_t pos =
+            RD_KAFKA_FETCH_POS(offset + 1, -1 /*no leader epoch known*/);
 
         /* Find toppar */
         rd_kafka_topic_rdlock(rkt);
@@ -646,7 +657,7 @@ rd_kafka_resp_err_t rd_kafka_offset_store(rd_kafka_topic_t *app_rkt,
         }
         rd_kafka_topic_rdunlock(rkt);
 
-        err = rd_kafka_offset_store0(rktp, offset + 1,
+        err = rd_kafka_offset_store0(rktp, pos, NULL, 0,
                                      rd_false /* Don't force */, RD_DO_LOCK);
 
         rd_kafka_toppar_destroy(rktp);
@@ -668,6 +679,8 @@ rd_kafka_offsets_store(rd_kafka_t *rk,
         for (i = 0; i < offsets->cnt; i++) {
                 rd_kafka_topic_partition_t *rktpar = &offsets->elems[i];
                 rd_kafka_toppar_t *rktp;
+                rd_kafka_fetch_pos_t pos =
+                    RD_KAFKA_FETCH_POS(rktpar->offset, -1);
 
                 rktp =
                     rd_kafka_topic_partition_get_toppar(rk, rktpar, rd_false);
@@ -677,9 +690,12 @@ rd_kafka_offsets_store(rd_kafka_t *rk,
                         continue;
                 }
 
-                rktpar->err = rd_kafka_offset_store0(rktp, rktpar->offset,
-                                                     rd_false /* don't force */,
-                                                     RD_DO_LOCK);
+                pos.leader_epoch =
+                    rd_kafka_topic_partition_get_leader_epoch(rktpar);
+
+                rktpar->err = rd_kafka_offset_store0(
+                    rktp, pos, rktpar->metadata, rktpar->metadata_size,
+                    rd_false /* don't force */, RD_DO_LOCK);
                 rd_kafka_toppar_destroy(rktp);
 
                 if (rktpar->err)
@@ -693,6 +709,39 @@ rd_kafka_offsets_store(rd_kafka_t *rk,
 }
 
 
+rd_kafka_error_t *rd_kafka_offset_store_message(rd_kafka_message_t *rkmessage) {
+        rd_kafka_toppar_t *rktp;
+        rd_kafka_op_t *rko;
+        rd_kafka_resp_err_t err;
+        rd_kafka_msg_t *rkm = (rd_kafka_msg_t *)rkmessage;
+        rd_kafka_fetch_pos_t pos;
+
+        if (rkmessage->err)
+                return rd_kafka_error_new(RD_KAFKA_RESP_ERR__INVALID_ARG,
+                                          "Message object must not have an "
+                                          "error set");
+
+        if (unlikely(!(rko = rd_kafka_message2rko(rkmessage)) ||
+                     !(rktp = rko->rko_rktp)))
+                return rd_kafka_error_new(RD_KAFKA_RESP_ERR__INVALID_ARG,
+                                          "Invalid message object, "
+                                          "not a consumed message");
+
+        pos = RD_KAFKA_FETCH_POS(rkmessage->offset + 1,
+                                 rkm->rkm_u.consumer.leader_epoch);
+        err = rd_kafka_offset_store0(rktp, pos, NULL, 0,
+                                     rd_false /* Don't force */, RD_DO_LOCK);
+
+        if (err == RD_KAFKA_RESP_ERR__STATE)
+                return rd_kafka_error_new(err, "Partition is not assigned");
+        else if (err)
+                return rd_kafka_error_new(err, "Failed to store offset: %s",
+                                          rd_kafka_err2str(err));
+
+        return NULL;
+}
+
+
 
 /**
  * Decommissions the use of an offset file for a toppar.
@@ -723,7 +772,7 @@ static rd_kafka_op_res_t rd_kafka_offset_reset_op_cb(rd_kafka_t *rk,
         rd_kafka_toppar_t *rktp = rko->rko_rktp;
         rd_kafka_toppar_lock(rktp);
         rd_kafka_offset_reset(rktp, rko->rko_u.offset_reset.broker_id,
-                              rko->rko_u.offset_reset.offset, rko->rko_err,
+                              rko->rko_u.offset_reset.pos, rko->rko_err, "%s",
                               rko->rko_u.offset_reset.reason);
         rd_kafka_toppar_unlock(rktp);
         return RD_KAFKA_OP_RES_HANDLED;
@@ -735,20 +784,27 @@ static rd_kafka_op_res_t rd_kafka_offset_reset_op_cb(rd_kafka_t *rk,
  *
  * @param rktp the toppar
  * @param broker_id Originating broker, if any, else RD_KAFKA_NODEID_UA.
- * @param err_offset a logical offset, or offset corresponding to the error.
+ * @param err_pos a logical offset, or offset corresponding to the error.
  * @param err the error, or RD_KAFKA_RESP_ERR_NO_ERROR if offset is logical.
- * @param reason a reason string for logging.
+ * @param fmt a reason string for logging.
  *
- * @locality: any. if not main thread, work will be enqued on main thread.
- * @ocks: toppar_lock() MUST be held
+ * @locality any. if not main thread, work will be enqued on main thread.
+ * @locks_required toppar_lock() MUST be held
  */
 void rd_kafka_offset_reset(rd_kafka_toppar_t *rktp,
                            int32_t broker_id,
-                           int64_t err_offset,
+                           rd_kafka_fetch_pos_t err_pos,
                            rd_kafka_resp_err_t err,
-                           const char *reason) {
-        int64_t offset    = RD_KAFKA_OFFSET_INVALID;
-        const char *extra = "";
+                           const char *fmt,
+                           ...) {
+        rd_kafka_fetch_pos_t pos = {RD_KAFKA_OFFSET_INVALID, -1};
+        const char *extra        = "";
+        char reason[512];
+        va_list ap;
+
+        va_start(ap, fmt);
+        rd_vsnprintf(reason, sizeof(reason), fmt, ap);
+        va_end(ap);
 
         /* Enqueue op for toppar handler thread if we're on the wrong thread. */
         if (!thrd_is_current(rktp->rktp_rkt->rkt_rk->rk_thread)) {
@@ -758,48 +814,49 @@ void rd_kafka_offset_reset(rd_kafka_toppar_t *rktp,
                 rko->rko_err                      = err;
                 rko->rko_rktp                     = rd_kafka_toppar_keep(rktp);
                 rko->rko_u.offset_reset.broker_id = broker_id;
-                rko->rko_u.offset_reset.offset    = err_offset;
+                rko->rko_u.offset_reset.pos       = err_pos;
                 rko->rko_u.offset_reset.reason    = rd_strdup(reason);
                 rd_kafka_q_enq(rktp->rktp_ops, rko);
                 return;
         }
 
-        if (err_offset == RD_KAFKA_OFFSET_INVALID || err)
-                offset = rktp->rktp_rkt->rkt_conf.auto_offset_reset;
+        if (err_pos.offset == RD_KAFKA_OFFSET_INVALID || err)
+                pos.offset = rktp->rktp_rkt->rkt_conf.auto_offset_reset;
         else
-                offset = err_offset;
+                pos.offset = err_pos.offset;
 
-        if (offset == RD_KAFKA_OFFSET_INVALID) {
+        if (pos.offset == RD_KAFKA_OFFSET_INVALID) {
                 /* Error, auto.offset.reset tells us to error out. */
                 if (broker_id != RD_KAFKA_NODEID_UA)
                         rd_kafka_consumer_err(
                             rktp->rktp_fetchq, broker_id,
                             RD_KAFKA_RESP_ERR__AUTO_OFFSET_RESET, 0, NULL, rktp,
-                            err_offset, "%s: %s (broker %" PRId32 ")", reason,
-                            rd_kafka_err2str(err), broker_id);
+                            err_pos.offset, "%s: %s (broker %" PRId32 ")",
+                            reason, rd_kafka_err2str(err), broker_id);
                 else
                         rd_kafka_consumer_err(
                             rktp->rktp_fetchq, broker_id,
                             RD_KAFKA_RESP_ERR__AUTO_OFFSET_RESET, 0, NULL, rktp,
-                            err_offset, "%s: %s", reason,
+                            err_pos.offset, "%s: %s", reason,
                             rd_kafka_err2str(err));
 
                 rd_kafka_toppar_set_fetch_state(rktp,
                                                 RD_KAFKA_TOPPAR_FETCH_NONE);
 
-        } else if (offset == RD_KAFKA_OFFSET_BEGINNING &&
+        } else if (pos.offset == RD_KAFKA_OFFSET_BEGINNING &&
                    rktp->rktp_lo_offset >= 0) {
                 /* Use cached log start from last Fetch if available.
                  * Note: The cached end offset (rktp_ls_offset) can't be
                  *       used here since the End offset is a constantly moving
                  *       target as new messages are produced. */
-                extra  = "cached BEGINNING offset ";
-                offset = rktp->rktp_lo_offset;
-                rd_kafka_toppar_next_offset_handle(rktp, offset);
+                extra            = "cached BEGINNING offset ";
+                pos.offset       = rktp->rktp_lo_offset;
+                pos.leader_epoch = -1;
+                rd_kafka_toppar_next_offset_handle(rktp, pos);
 
         } else {
                 /* Else query cluster for offset */
-                rktp->rktp_query_offset = offset;
+                rktp->rktp_query_pos = pos;
                 rd_kafka_toppar_set_fetch_state(
                     rktp, RD_KAFKA_TOPPAR_FETCH_OFFSET_QUERY);
         }
@@ -808,37 +865,340 @@ void rd_kafka_offset_reset(rd_kafka_toppar_t *rktp,
          * critical impact. For non-errors, or for auto.offset.reset=error,
          * the reason is simply debug-logged. */
         if (!err || err == RD_KAFKA_RESP_ERR__NO_OFFSET ||
-            offset == RD_KAFKA_OFFSET_INVALID)
+            pos.offset == RD_KAFKA_OFFSET_INVALID)
                 rd_kafka_dbg(
                     rktp->rktp_rkt->rkt_rk, TOPIC, "OFFSET",
-                    "%s [%" PRId32
-                    "]: offset reset (at offset %s, broker %" PRId32
+                    "%s [%" PRId32 "]: offset reset (at %s, broker %" PRId32
                     ") "
                     "to %s%s: %s: %s",
                     rktp->rktp_rkt->rkt_topic->str, rktp->rktp_partition,
-                    rd_kafka_offset2str(err_offset), broker_id, extra,
-                    rd_kafka_offset2str(offset), reason, rd_kafka_err2str(err));
+                    rd_kafka_fetch_pos2str(err_pos), broker_id, extra,
+                    rd_kafka_fetch_pos2str(pos), reason, rd_kafka_err2str(err));
         else
                 rd_kafka_log(
                     rktp->rktp_rkt->rkt_rk, LOG_WARNING, "OFFSET",
-                    "%s [%" PRId32
-                    "]: offset reset (at offset %s, broker %" PRId32
-                    ") "
-                    "to %s%s: %s: %s",
+                    "%s [%" PRId32 "]: offset reset (at %s, broker %" PRId32
+                    ") to %s%s: %s: %s",
                     rktp->rktp_rkt->rkt_topic->str, rktp->rktp_partition,
-                    rd_kafka_offset2str(err_offset), broker_id, extra,
-                    rd_kafka_offset2str(offset), reason, rd_kafka_err2str(err));
+                    rd_kafka_fetch_pos2str(err_pos), broker_id, extra,
+                    rd_kafka_fetch_pos2str(pos), reason, rd_kafka_err2str(err));
 
         /* Note: If rktp is not delegated to the leader, then low and high
            offsets will necessarily be cached from the last FETCH request,
            and so this offset query will never occur in that case for
            BEGINNING / END logical offsets. */
         if (rktp->rktp_fetch_state == RD_KAFKA_TOPPAR_FETCH_OFFSET_QUERY)
-                rd_kafka_toppar_offset_request(rktp, rktp->rktp_query_offset,
+                rd_kafka_toppar_offset_request(rktp, rktp->rktp_query_pos,
                                                err ? 100 : 0);
 }
 
 
+
+/**
+ * @brief Offset validation retry timer
+ */
+static void rd_kafka_offset_validate_tmr_cb(rd_kafka_timers_t *rkts,
+                                            void *arg) {
+        rd_kafka_toppar_t *rktp = arg;
+
+        rd_kafka_toppar_lock(rktp);
+        rd_kafka_offset_validate(rktp, "retrying offset validation");
+        rd_kafka_toppar_unlock(rktp);
+}
+
+
+
+/**
+ * @brief OffsetForLeaderEpochResponse handler that
+ *        pushes the matched toppar's to the next state.
+ *
+ * @locality rdkafka main thread
+ */
+static void rd_kafka_toppar_handle_OffsetForLeaderEpoch(rd_kafka_t *rk,
+                                                        rd_kafka_broker_t *rkb,
+                                                        rd_kafka_resp_err_t err,
+                                                        rd_kafka_buf_t *rkbuf,
+                                                        rd_kafka_buf_t *request,
+                                                        void *opaque) {
+        rd_kafka_topic_partition_list_t *parts = NULL;
+        rd_kafka_toppar_t *rktp                = opaque;
+        rd_kafka_topic_partition_t *rktpar;
+        int64_t end_offset;
+        int32_t end_offset_leader_epoch;
+
+        if (err == RD_KAFKA_RESP_ERR__DESTROY) {
+                rd_kafka_toppar_destroy(rktp); /* Drop refcnt */
+                return;
+        }
+
+        err = rd_kafka_handle_OffsetForLeaderEpoch(rk, rkb, err, rkbuf, request,
+                                                   &parts);
+
+        rd_kafka_toppar_lock(rktp);
+
+        if (rktp->rktp_fetch_state != RD_KAFKA_TOPPAR_FETCH_VALIDATE_EPOCH_WAIT)
+                err = RD_KAFKA_RESP_ERR__OUTDATED;
+
+        if (unlikely(!err && parts->cnt == 0))
+                err = RD_KAFKA_RESP_ERR__UNKNOWN_PARTITION;
+
+        if (!err) {
+                err = (&parts->elems[0])->err;
+        }
+
+        if (err) {
+                int actions;
+
+                rd_rkb_dbg(rkb, FETCH, "OFFSETVALID",
+                           "%.*s [%" PRId32
+                           "]: OffsetForLeaderEpoch requested failed: %s",
+                           RD_KAFKAP_STR_PR(rktp->rktp_rkt->rkt_topic),
+                           rktp->rktp_partition, rd_kafka_err2str(err));
+
+                if (err == RD_KAFKA_RESP_ERR__UNSUPPORTED_FEATURE) {
+                        rd_rkb_dbg(rkb, FETCH, "VALIDATE",
+                                   "%.*s [%" PRId32
+                                   "]: offset and epoch validation not "
+                                   "supported by broker: validation skipped",
+                                   RD_KAFKAP_STR_PR(rktp->rktp_rkt->rkt_topic),
+                                   rktp->rktp_partition);
+                        rd_kafka_toppar_set_fetch_state(
+                            rktp, RD_KAFKA_TOPPAR_FETCH_ACTIVE);
+                        goto done;
+
+                } else if (err == RD_KAFKA_RESP_ERR__OUTDATED) {
+                        /* Partition state has changed, this response
+                         * is outdated. */
+                        goto done;
+                }
+
+                actions = rd_kafka_err_action(
+                    rkb, err, request, RD_KAFKA_ERR_ACTION_REFRESH,
+                    RD_KAFKA_RESP_ERR_UNKNOWN_LEADER_EPOCH,
+                    RD_KAFKA_ERR_ACTION_REFRESH,
+                    RD_KAFKA_RESP_ERR_FENCED_LEADER_EPOCH,
+                    RD_KAFKA_ERR_ACTION_REFRESH,
+                    RD_KAFKA_RESP_ERR_UNKNOWN_TOPIC_OR_PART,
+                    RD_KAFKA_ERR_ACTION_REFRESH,
+                    RD_KAFKA_RESP_ERR_OFFSET_NOT_AVAILABLE,
+                    RD_KAFKA_ERR_ACTION_REFRESH,
+                    RD_KAFKA_RESP_ERR_KAFKA_STORAGE_ERROR,
+                    RD_KAFKA_ERR_ACTION_END);
+
+
+                if (actions & RD_KAFKA_ERR_ACTION_REFRESH)
+                        /* Metadata refresh is ongoing, so force it */
+                        rd_kafka_topic_leader_query0(rk, rktp->rktp_rkt, 1,
+                                                     rd_true /* force */);
+
+                /* No need for refcnt on rktp for timer opaque
+                 * since the timer resides on the rktp and will be
+                 * stopped on toppar remove.
+                 * Retries the validation with a new call even in
+                 * case of permanent error. */
+                rd_kafka_timer_start_oneshot(
+                    &rk->rk_timers, &rktp->rktp_validate_tmr, rd_false,
+                    500 * 1000 /* 500ms */, rd_kafka_offset_validate_tmr_cb,
+                    rktp);
+                goto done;
+        }
+
+
+        rktpar     = &parts->elems[0];
+        end_offset = rktpar->offset;
+        end_offset_leader_epoch =
+            rd_kafka_topic_partition_get_leader_epoch(rktpar);
+
+        if (end_offset < 0 || end_offset_leader_epoch < 0) {
+                rd_kafka_offset_reset(
+                    rktp, rd_kafka_broker_id(rkb),
+                    rktp->rktp_offset_validation_pos,
+                    RD_KAFKA_RESP_ERR__LOG_TRUNCATION,
+                    "No epoch found less or equal to "
+                    "%s: broker end offset is %" PRId64
+                    " (offset leader epoch %" PRId32
+                    ")."
+                    " Reset using configured policy.",
+                    rd_kafka_fetch_pos2str(rktp->rktp_offset_validation_pos),
+                    end_offset, end_offset_leader_epoch);
+
+        } else if (end_offset < rktp->rktp_offset_validation_pos.offset) {
+
+                if (rktp->rktp_rkt->rkt_conf.auto_offset_reset ==
+                    RD_KAFKA_OFFSET_INVALID /* auto.offset.reset=error */) {
+                        rd_kafka_offset_reset(
+                            rktp, rd_kafka_broker_id(rkb),
+                            RD_KAFKA_FETCH_POS(RD_KAFKA_OFFSET_INVALID,
+                                               rktp->rktp_leader_epoch),
+                            RD_KAFKA_RESP_ERR__LOG_TRUNCATION,
+                            "Partition log truncation detected at %s: "
+                            "broker end offset is %" PRId64
+                            " (offset leader epoch %" PRId32
+                            "). "
+                            "Reset to INVALID.",
+                            rd_kafka_fetch_pos2str(
+                                rktp->rktp_offset_validation_pos),
+                            end_offset, end_offset_leader_epoch);
+
+                } else {
+                        rd_kafka_toppar_unlock(rktp);
+
+                        /* Seek to the updated end offset */
+                        rd_kafka_fetch_pos_t fetch_pos =
+                            rd_kafka_topic_partition_get_fetch_pos(rktpar);
+                        fetch_pos.validated = rd_true;
+
+                        rd_kafka_toppar_op_seek(rktp, fetch_pos,
+                                                RD_KAFKA_NO_REPLYQ);
+
+                        rd_kafka_topic_partition_list_destroy(parts);
+                        rd_kafka_toppar_destroy(rktp);
+
+                        return;
+                }
+
+        } else {
+                rd_rkb_dbg(rkb, FETCH, "OFFSETVALID",
+                           "%.*s [%" PRId32
+                           "]: offset and epoch validation "
+                           "succeeded: broker end offset %" PRId64
+                           " (offset leader epoch %" PRId32 ")",
+                           RD_KAFKAP_STR_PR(rktp->rktp_rkt->rkt_topic),
+                           rktp->rktp_partition, end_offset,
+                           end_offset_leader_epoch);
+
+                rd_kafka_toppar_set_fetch_state(rktp,
+                                                RD_KAFKA_TOPPAR_FETCH_ACTIVE);
+        }
+
+done:
+        rd_kafka_toppar_unlock(rktp);
+
+        if (parts)
+                rd_kafka_topic_partition_list_destroy(parts);
+        rd_kafka_toppar_destroy(rktp);
+}
+
+
+static rd_kafka_op_res_t rd_kafka_offset_validate_op_cb(rd_kafka_t *rk,
+                                                        rd_kafka_q_t *rkq,
+                                                        rd_kafka_op_t *rko) {
+        rd_kafka_toppar_t *rktp = rko->rko_rktp;
+        rd_kafka_toppar_lock(rktp);
+        rd_kafka_offset_validate(rktp, "%s", rko->rko_u.offset_reset.reason);
+        rd_kafka_toppar_unlock(rktp);
+        return RD_KAFKA_OP_RES_HANDLED;
+}
+
+/**
+ * @brief Validate partition epoch and offset (KIP-320).
+ *
+ * @param rktp the toppar
+ * @param err Optional error code that triggered the validation.
+ * @param fmt a reason string for logging.
+ *
+ * @locality any. if not main thread, work will be enqued on main thread.
+ * @locks_required toppar_lock() MUST be held
+ */
+void rd_kafka_offset_validate(rd_kafka_toppar_t *rktp, const char *fmt, ...) {
+        rd_kafka_topic_partition_list_t *parts;
+        rd_kafka_topic_partition_t *rktpar;
+        char reason[512];
+        va_list ap;
+
+        if (rktp->rktp_rkt->rkt_rk->rk_type != RD_KAFKA_CONSUMER)
+                return;
+
+        va_start(ap, fmt);
+        rd_vsnprintf(reason, sizeof(reason), fmt, ap);
+        va_end(ap);
+
+        /* Enqueue op for toppar handler thread if we're on the wrong thread. */
+        if (!thrd_is_current(rktp->rktp_rkt->rkt_rk->rk_thread)) {
+                /* Reuse OP_OFFSET_RESET type */
+                rd_kafka_op_t *rko =
+                    rd_kafka_op_new(RD_KAFKA_OP_OFFSET_RESET | RD_KAFKA_OP_CB);
+                rko->rko_op_cb                 = rd_kafka_offset_validate_op_cb;
+                rko->rko_rktp                  = rd_kafka_toppar_keep(rktp);
+                rko->rko_u.offset_reset.reason = rd_strdup(reason);
+                rd_kafka_q_enq(rktp->rktp_ops, rko);
+                return;
+        }
+
+        if (rktp->rktp_fetch_state != RD_KAFKA_TOPPAR_FETCH_ACTIVE &&
+            rktp->rktp_fetch_state !=
+                RD_KAFKA_TOPPAR_FETCH_VALIDATE_EPOCH_WAIT) {
+                rd_kafka_dbg(rktp->rktp_rkt->rkt_rk, FETCH, "VALIDATE",
+                             "%.*s [%" PRId32
+                             "]: skipping offset "
+                             "validation in fetch state %s",
+                             RD_KAFKAP_STR_PR(rktp->rktp_rkt->rkt_topic),
+                             rktp->rktp_partition,
+                             rd_kafka_fetch_states[rktp->rktp_fetch_state]);
+                return;
+        }
+
+
+        if (rktp->rktp_leader_id == -1 || !rktp->rktp_leader ||
+            rktp->rktp_leader->rkb_source == RD_KAFKA_INTERNAL) {
+                rd_kafka_dbg(rktp->rktp_rkt->rkt_rk, FETCH, "VALIDATE",
+                             "%.*s [%" PRId32
+                             "]: unable to perform offset "
+                             "validation: partition leader not available",
+                             RD_KAFKAP_STR_PR(rktp->rktp_rkt->rkt_topic),
+                             rktp->rktp_partition);
+
+                rd_kafka_toppar_set_fetch_state(rktp,
+                                                RD_KAFKA_TOPPAR_FETCH_ACTIVE);
+                return;
+        }
+
+        /* If the fetch start position does not have an epoch set then
+         * there is no point in doing validation.
+         * This is the case for epoch-less seek()s or epoch-less
+         * committed offsets. */
+        if (rktp->rktp_offset_validation_pos.leader_epoch == -1) {
+                rd_kafka_dbg(
+                    rktp->rktp_rkt->rkt_rk, FETCH, "VALIDATE",
+                    "%.*s [%" PRId32
+                    "]: skipping offset "
+                    "validation for %s: no leader epoch set",
+                    RD_KAFKAP_STR_PR(rktp->rktp_rkt->rkt_topic),
+                    rktp->rktp_partition,
+                    rd_kafka_fetch_pos2str(rktp->rktp_offset_validation_pos));
+                rd_kafka_toppar_set_fetch_state(rktp,
+                                                RD_KAFKA_TOPPAR_FETCH_ACTIVE);
+                return;
+        }
+
+        rd_kafka_toppar_set_fetch_state(
+            rktp, RD_KAFKA_TOPPAR_FETCH_VALIDATE_EPOCH_WAIT);
+
+        /* Construct and send OffsetForLeaderEpochRequest */
+        parts  = rd_kafka_topic_partition_list_new(1);
+        rktpar = rd_kafka_topic_partition_list_add(
+            parts, rktp->rktp_rkt->rkt_topic->str, rktp->rktp_partition);
+        rd_kafka_topic_partition_set_leader_epoch(
+            rktpar, rktp->rktp_offset_validation_pos.leader_epoch);
+        rd_kafka_topic_partition_set_current_leader_epoch(
+            rktpar, rktp->rktp_leader_epoch);
+        rd_kafka_toppar_keep(rktp); /* for request opaque */
+
+        rd_rkb_dbg(
+            rktp->rktp_leader, FETCH, "VALIDATE",
+            "%.*s [%" PRId32
+            "]: querying broker for epoch "
+            "validation of %s: %s",
+            RD_KAFKAP_STR_PR(rktp->rktp_rkt->rkt_topic), rktp->rktp_partition,
+            rd_kafka_fetch_pos2str(rktp->rktp_offset_validation_pos), reason);
+
+        rd_kafka_OffsetForLeaderEpochRequest(
+            rktp->rktp_leader, parts, RD_KAFKA_REPLYQ(rktp->rktp_ops, 0),
+            rd_kafka_toppar_handle_OffsetForLeaderEpoch, rktp);
+        rd_kafka_topic_partition_list_destroy(parts);
+}
+
+
 /**
  * Escape any special characters in filename 'in' and write escaped
  * string to 'out' (of max size out_size).
@@ -953,15 +1313,16 @@ static void rd_kafka_offset_file_init(rd_kafka_toppar_t *rktp) {
 
         if (offset != RD_KAFKA_OFFSET_INVALID) {
                 /* Start fetching from offset */
-                rktp->rktp_stored_offset    = offset;
-                rktp->rktp_committed_offset = offset;
-                rd_kafka_toppar_next_offset_handle(rktp, offset);
+                rktp->rktp_stored_pos.offset    = offset;
+                rktp->rktp_committed_pos.offset = offset;
+                rd_kafka_toppar_next_offset_handle(rktp, rktp->rktp_stored_pos);
 
         } else {
                 /* Offset was not usable: perform offset reset logic */
-                rktp->rktp_committed_offset = RD_KAFKA_OFFSET_INVALID;
+                rktp->rktp_committed_pos.offset = RD_KAFKA_OFFSET_INVALID;
                 rd_kafka_offset_reset(
-                    rktp, RD_KAFKA_NODEID_UA, RD_KAFKA_OFFSET_INVALID,
+                    rktp, RD_KAFKA_NODEID_UA,
+                    RD_KAFKA_FETCH_POS(RD_KAFKA_OFFSET_INVALID, -1),
                     RD_KAFKA_RESP_ERR__FS, "non-readable offset file");
         }
 }
@@ -978,14 +1339,16 @@ rd_kafka_offset_broker_term(rd_kafka_toppar_t *rktp) {
 
 
 /**
- * Prepare a toppar for using broker offset commit (broker 0.8.2 or later).
- * When using KafkaConsumer (high-level consumer) this functionality is
- * disabled in favour of the cgrp commits for the entire set of subscriptions.
+ * Prepare a toppar for using broker offset commit (broker 0.8.2 or
+ * later). When using KafkaConsumer (high-level consumer) this
+ * functionality is disabled in favour of the cgrp commits for the
+ * entire set of subscriptions.
  */
 static void rd_kafka_offset_broker_init(rd_kafka_toppar_t *rktp) {
         if (!rd_kafka_is_simple_consumer(rktp->rktp_rkt->rkt_rk))
                 return;
-        rd_kafka_offset_reset(rktp, RD_KAFKA_NODEID_UA, RD_KAFKA_OFFSET_STORED,
+        rd_kafka_offset_reset(rktp, RD_KAFKA_NODEID_UA,
+                              RD_KAFKA_FETCH_POS(RD_KAFKA_OFFSET_STORED, -1),
                               RD_KAFKA_RESP_ERR_NO_ERROR,
                               "query broker for offsets");
 }
@@ -1055,23 +1418,27 @@ rd_kafka_resp_err_t rd_kafka_offset_store_stop(rd_kafka_toppar_t *rktp) {
         rd_kafka_dbg(rktp->rktp_rkt->rkt_rk, TOPIC, "OFFSET",
                      "%s [%" PRId32
                      "]: stopping offset store "
-                     "(stored offset %" PRId64 ", committed offset %" PRId64
-                     ", EOF offset %" PRId64 ")",
+                     "(stored %s, committed %s, EOF offset %" PRId64 ")",
                      rktp->rktp_rkt->rkt_topic->str, rktp->rktp_partition,
-                     rktp->rktp_stored_offset, rktp->rktp_committed_offset,
+                     rd_kafka_fetch_pos2str(rktp->rktp_stored_pos),
+                     rd_kafka_fetch_pos2str(rktp->rktp_committed_pos),
                      rktp->rktp_offsets_fin.eof_offset);
 
         /* Store end offset for empty partitions */
         if (rktp->rktp_rkt->rkt_rk->rk_conf.enable_auto_offset_store &&
-            rktp->rktp_stored_offset == RD_KAFKA_OFFSET_INVALID &&
+            rktp->rktp_stored_pos.offset == RD_KAFKA_OFFSET_INVALID &&
             rktp->rktp_offsets_fin.eof_offset > 0)
-                rd_kafka_offset_store0(rktp, rktp->rktp_offsets_fin.eof_offset,
-                                       rd_true /* force */, RD_DONT_LOCK);
+                rd_kafka_offset_store0(
+                    rktp,
+                    RD_KAFKA_FETCH_POS(rktp->rktp_offsets_fin.eof_offset,
+                                       rktp->rktp_leader_epoch),
+                    NULL, 0, rd_true /* force */, RD_DONT_LOCK);
 
         /* Commit offset to backing store.
          * This might be an async operation. */
         if (rd_kafka_is_simple_consumer(rktp->rktp_rkt->rkt_rk) &&
-            rktp->rktp_stored_offset > rktp->rktp_committed_offset)
+            rd_kafka_fetch_pos_cmp(&rktp->rktp_stored_pos,
+                                   &rktp->rktp_committed_pos) > 0)
                 err = rd_kafka_offset_commit(rktp, "offset store stop");
 
         /* If stop is in progress (async commit), return now. */
@@ -1097,12 +1464,11 @@ void rd_kafka_offset_query_tmr_cb(rd_kafka_timers_t *rkts, void *arg) {
         rd_kafka_toppar_lock(rktp);
         rd_kafka_dbg(rktp->rktp_rkt->rkt_rk, TOPIC, "OFFSET",
                      "Topic %s [%" PRId32
-                     "]: timed offset query for %s in "
-                     "state %s",
+                     "]: timed offset query for %s in state %s",
                      rktp->rktp_rkt->rkt_topic->str, rktp->rktp_partition,
-                     rd_kafka_offset2str(rktp->rktp_query_offset),
+                     rd_kafka_fetch_pos2str(rktp->rktp_query_pos),
                      rd_kafka_fetch_states[rktp->rktp_fetch_state]);
-        rd_kafka_toppar_offset_request(rktp, rktp->rktp_query_offset, 0);
+        rd_kafka_toppar_offset_request(rktp, rktp->rktp_query_pos, 0);
         rd_kafka_toppar_unlock(rktp);
 }
 
@@ -1121,7 +1487,7 @@ void rd_kafka_offset_store_init(rd_kafka_toppar_t *rktp) {
                      store_names[rktp->rktp_rkt->rkt_conf.offset_store_method]);
 
         /* The committed offset is unknown at this point. */
-        rktp->rktp_committed_offset = RD_KAFKA_OFFSET_INVALID;
+        rktp->rktp_committed_pos.offset = RD_KAFKA_OFFSET_INVALID;
 
         /* Set up the commit interval (for simple consumer). */
         if (rd_kafka_is_simple_consumer(rktp->rktp_rkt->rkt_rk) &&
@@ -1148,3 +1514,26 @@ void rd_kafka_offset_store_init(rd_kafka_toppar_t *rktp) {
 
         rktp->rktp_flags |= RD_KAFKA_TOPPAR_F_OFFSET_STORE;
 }
+
+
+/**
+ * Update toppar app_pos and store_offset (if enabled) to the provided
+ * offset and epoch.
+ */
+void rd_kafka_update_app_pos(rd_kafka_t *rk,
+                             rd_kafka_toppar_t *rktp,
+                             rd_kafka_fetch_pos_t pos,
+                             rd_dolock_t do_lock) {
+
+        if (do_lock)
+                rd_kafka_toppar_lock(rktp);
+
+        rktp->rktp_app_pos = pos;
+        if (rk->rk_conf.enable_auto_offset_store)
+                rd_kafka_offset_store0(rktp, pos, NULL, 0,
+                                       /* force: ignore assignment state */
+                                       rd_true, RD_DONT_LOCK);
+
+        if (do_lock)
+                rd_kafka_toppar_unlock(rktp);
+}
