@@ -141,7 +141,14 @@ typedef enum {
         RD_KAFKA_OP_DESCRIBECONSUMERGROUPS,  /**< Admin:
                                               *   DescribeConsumerGroups
                                               *   u.admin_request */
-        RD_KAFKA_OP_DELETEGROUPS, /**< Admin: DeleteGroups: u.admin_request*/
+        RD_KAFKA_OP_DESCRIBECLUSTER,         /**< Admin:
+                                              *   DescribeCluster
+                                              *   u.admin_request */
+
+        RD_KAFKA_OP_DESCRIBETOPICS, /**< Admin:
+                                     *   DescribeTopics
+                                     *   u.admin_request */
+        RD_KAFKA_OP_DELETEGROUPS,   /**< Admin: DeleteGroups: u.admin_request*/
         RD_KAFKA_OP_DELETECONSUMERGROUPOFFSETS, /**< Admin:
                                                  *   DeleteConsumerGroupOffsets
                                                  *   u.admin_request */
@@ -172,6 +179,7 @@ typedef enum {
         RD_KAFKA_OP_ALTERUSERSCRAMCREDENTIALS,    /* < Admin:
                                                      AlterUserScramCredentials
                                                      u.admin_request >*/
+        RD_KAFKA_OP_LISTOFFSETS, /**< Admin: ListOffsets u.admin_request >*/
         RD_KAFKA_OP__END
 } rd_kafka_op_type_t;
 
@@ -522,6 +530,9 @@ struct rd_kafka_op_s {
                         char *errstr; /**< Error string, if rko_err
                                        *   is set, else NULL. */
 
+                        /** Result cb for this op */
+                        void (*result_cb)(rd_kafka_op_t *);
+
                         rd_list_t results; /**< Type depends on request type:
                                             *
                                             * (rd_kafka_topic_result_t *):
@@ -725,7 +736,7 @@ rd_kafka_op_t *rd_kafka_op_new_fetch_msg(rd_kafka_msg_t **rkmp,
                                          rd_kafka_toppar_t *rktp,
                                          int32_t version,
                                          rd_kafka_buf_t *rkbuf,
-                                         int64_t offset,
+                                         rd_kafka_fetch_pos_t pos,
                                          size_t key_len,
                                          const void *key,
                                          size_t val_len,
@@ -734,7 +745,7 @@ rd_kafka_op_t *rd_kafka_op_new_fetch_msg(rd_kafka_msg_t **rkmp,
 rd_kafka_op_t *rd_kafka_op_new_ctrl_msg(rd_kafka_toppar_t *rktp,
                                         int32_t version,
                                         rd_kafka_buf_t *rkbuf,
-                                        int64_t offset);
+                                        rd_kafka_fetch_pos_t pos);
 
 void rd_kafka_op_throttle_time(struct rd_kafka_broker_s *rkb,
                                rd_kafka_q_t *rkq,
